#!/usr/bin/env python
#
# Copyright (c) 2015, Linaro Limited
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from collections import namedtuple
PublicKey = namedtuple("PublicKey", "e n")

def importKey_crypto(pem_key_file):
	try:
		module = __import__("Crypto.PublicKey.RSA")
		f = open(pem_key_file, 'r')
		key = module.PublicKey.RSA.importKey(f.read())
		f.close()
		return PublicKey(e=key.publickey().e, n=module.Util.number.long_to_bytes(key.publickey().n))
	except ImportError:
		return None

def importKey_openssl(pem_key_file):
	import subprocess
	cmd = "cat " + pem_key_file + " | openssl rsa -inform PEM -noout -text | grep publicE | sed 's/publicExponent: //' | cut -d ' ' -f1 | tr -d '\n'"
	e = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
	if not e:
		print "Exporting exponent value from key {0} has failed.".format(pem_key_file)
                return None
	cmd = "cat " + pem_key_file + " | openssl rsa -inform PEM -noout -modulus | sed 's/Modulus=//' | tr -d '\n'"
	n = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
	if not n:
		print "Exporting modulus value from key {0} has failed.".format(pem_key_file)
		return None
	return PublicKey(e=e, n=n.decode("hex"))

def get_args():
	import argparse

	parser = argparse.ArgumentParser()
	parser.add_argument('--prefix', required=True, \
		help='Prefix for the public key exponent and modulus in c file')

	parser.add_argument('--out', required=True, \
		help='Name of c file for the public key')

	parser.add_argument('--key', required=True, help='Name of key file')

	return parser.parse_args()

def main():
	import array

	args = get_args();

	key = importKey_crypto(args.key)
	if key is None:
		key = importKey_openssl(args.key)
		if key is None:
			return

	f = open(args.out, 'w')

	f.write("#include <stdint.h>\n");
	f.write("#include <stddef.h>\n\n");

	f.write("const uint32_t " + args.prefix + "_exponent = " +
		str(key.e) + ";\n\n")

	f.write("const uint8_t " + args.prefix + "_modulus[] = {\n")
	i = 0;
	for x in array.array("B", key.n):
		f.write("0x" + '{0:02x}'.format(x) + ",")
		i = i + 1;
		if i % 8 == 0:
			f.write("\n");
		else:
			f.write(" ");
	f.write("};\n");

	f.write("const size_t " + args.prefix + "_modulus_size = sizeof(" + \
		args.prefix + "_modulus);\n")

	f.close()

if __name__ == "__main__":
	main()
